package MT::Plugin::LastLogin;
use strict;
use warnings;
use base 'MT::Plugin';

use MT::Author qw( AUTHOR );
use MT::Session;

our $VERSION = '1.00';
our $NAME = ( split /::/, __PACKAGE__ )[-1];

my $plugin = __PACKAGE__->new(
    {   name        => $NAME,
        id          => lc $NAME,
        key         => lc $NAME,
        l10n_class  => $NAME . '::L10N',
        version     => $VERSION,
        author_name => 'SKYARC System Co., Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
        description => '<__trans phrase="Save user last login datetime.">',
    }
);
MT->add_plugin($plugin);

sub init_registry {
    my ($p) = @_;
    my $pkg = '$' . $NAME . '::' . $NAME;
    $p->registry(
        {   object_types => {
                author => {
                    last_session      => 'integer meta',
                    last_login        => 'integer meta',
                    last_failed_login => 'integer meta',
                },
            },
            list_properties => { author    => $pkg . '::ListProps::author', },
            callbacks       => { take_down => \&_take_down, },
        }
    );
}

sub _take_down {
    my $app = MT->app();

    my $user = $app->user();
    if ($user) {
        $user->last_session(time);

        my $user_id = $user->id();

        my @sess = MT::Session->load(
            { kind => 'US', },
            { sort => 'start', direction => 'descend', },
        );
        my @grep_sess = grep { $_->get('author_id') == $user_id } @sess;

        if ( $grep_sess[0] ) {
            $user->last_login( $grep_sess[0]->start() );
        }
    }
    else {
        my $username = $app->param('username');
        if ( defined $username && $username ne '' ) {
            $user = $app->user_class->load(
                {   name      => $username,
                    type      => AUTHOR,
                    auth_type => 'MT',
                }
            );
        }

        if ($user) {
            $user->last_failed_login(time);
        }
    }

    if ($user) {
        $user->update()
            or die $app->error(
            $NAME . ' : Cannot save last_session of MT::Author.' );
    }
}

1;
__END__
