package LastLogin::ListProps;
use strict;
use warnings;

use MT::Util qw( offset_time_list );

sub author {
    return {
        last_session => {
            label     => 'Last Session',
            display   => 'default',
            raw       => \&_raw_session,
            bulk_sort => \&_bs_session,
            order     => 1000,
        },
        last_login => {
            label     => 'Last Login',
            display   => 'default',
            raw       => \&_raw_login,
            bulk_sort => \&_bs_login,
            order     => 1010,
        },
        last_failed_login => {
            label     => 'Last Failed Login',
            display   => 'default',
            raw       => \&_raw_failed_login,
            bulk_sort => \&_bs_failed_login,
            order     => 1020,
        },
    };
}

sub _raw_session {
    _raw( $_[0], $_[1], 'last_session' );
}

sub _raw_login {
    _raw( $_[0], $_[1], 'last_login' );
}

sub _raw_failed_login {
    _raw( $_[0], $_[1], 'last_failed_login' );
}

sub _raw {
    my ( $prop, $obj, $col ) = @_;

    my $last_datetime = $obj->$col;
    if ( !$last_datetime ) {
        return $last_datetime;
    }

    my @ts = offset_time_list($last_datetime);
    my $ts = sprintf '%04d-%02d-%02d %02d:%02d:%02d',
        $ts[5] + 1900, $ts[4] + 1, @ts[ 3, 2, 1, 0 ];

    return $ts;
}

sub _bs_session {
    _bulk_sort( $_[0], $_[1], 'last_session' );
}

sub _bs_login {
    _bulk_sort( $_[0], $_[1], 'last_login' );
}

sub _bs_failed_login {
    _bulk_sort( $_[0], $_[1], 'last_failed_login' );
}

sub _bulk_sort {
    my ( $prop, $objs, $col ) = @_;
    return sort { $a->$col <=> $b->$col } @$objs;
}

1;
__END__
